/*
Copyright (C) 2001,2006 StrmnNrmn

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "stdafx.h"

#include "PSPRenderer.h"
#include "Texture.h"
#include "NativeTexture.h"
#include "RDP.h"
#include "PSPColour.h"

#include "Core\ROM.h"

#include "DaedIO.h"
#include "Debug\Dump.h"

#include <pspgu.h>

#define GL_TRUE                           1
#define GL_FALSE                          0

const u32 COL32_BLACK( MAKE_COLOUR_U32( 0,0,0 ) );
const u32 COL32_WHITE( MAKE_COLOUR_U32( 255, 255, 255 ) );


static const char * sc_colcombtypes32[32] =
{
	"Combined    ", "Texel0      ",
	"Texel1      ", "Primitive   ", 
	"Shade       ", "Env         ",
	"1           ", "CombAlp     ",
	"Texel0_Alp  ", "Texel1_Alp  ",
	"Prim_Alpha  ", "Shade_Alpha ",
	"Env_Alpha   ", "LOD_Frac    ",
	"PrimLODFrac ", "K5          ",
	"?           ", "?           ",
	"?           ", "?           ",
	"?           ", "?           ",
	"?           ", "?           ",
	"?           ", "?           ",
	"?           ", "?           ",
	"?           ", "?           ",
	"?           ",	"0           "
};
static const char *sc_colcombtypes16[16] =
{
	"Combined    ", "Texel0      ",
	"Texel1      ", "Primitive   ", 
	"Shade       ", "Env         ",
	"1           ", "CombAlp     ",
	"Texel0_Alp  ", "Texel1_Alp  ",
	"Prim_Alp    ", "Shade_Alpha ",
	"Env_Alpha   ", "LOD_Frac    ",
	"PrimLOD_Frac", "0           "
};
static const char *sc_colcombtypes8[8] =
{
	"Combined    ", "Texel0      ",
	"Texel1      ", "Primitive   ", 
	"Shade       ", "Env         ",
	"1           ", "0           ",
};



void PrintMux( FILE * fh, u64 mux )
{
	u32 mux0 = (u32)(mux>>32);
	u32 mux1 = (u32)(mux);
	
	u32 aRGB0  = (mux0>>20)&0x0F;	// c1 c1		// a0
	u32 bRGB0  = (mux1>>28)&0x0F;	// c1 c2		// b0
	u32 cRGB0  = (mux0>>15)&0x1F;	// c1 c3		// c0
	u32 dRGB0  = (mux1>>15)&0x07;	// c1 c4		// d0

	u32 aA0    = (mux0>>12)&0x07;	// c1 a1		// Aa0
	u32 bA0    = (mux1>>12)&0x07;	// c1 a2		// Ab0
	u32 cA0    = (mux0>>9 )&0x07;	// c1 a3		// Ac0
	u32 dA0    = (mux1>>9 )&0x07;	// c1 a4		// Ad0

	u32 aRGB1  = (mux0>>5 )&0x0F;	// c2 c1		// a1
	u32 bRGB1  = (mux1>>24)&0x0F;	// c2 c2		// b1
	u32 cRGB1  = (mux0    )&0x1F;	// c2 c3		// c1
	u32 dRGB1  = (mux1>>6 )&0x07;	// c2 c4		// d1
	
	u32 aA1    = (mux1>>21)&0x07;	// c2 a1		// Aa1
	u32 bA1    = (mux1>>3 )&0x07;	// c2 a2		// Ab1
	u32 cA1    = (mux1>>18)&0x07;	// c2 a3		// Ac1
	u32 dA1    = (mux1    )&0x07;	// c2 a4		// Ad1

	fprintf(fh, "\n\t\tcase 0x%08x%08xLL:\n", mux0, mux1);
	fprintf(fh, "\t\t//aRGB0: (%s - %s) * %s + %s\n", sc_colcombtypes16[aRGB0], sc_colcombtypes16[bRGB0], sc_colcombtypes32[cRGB0], sc_colcombtypes8[dRGB0]);		
	fprintf(fh, "\t\t//aA0  : (%s - %s) * %s + %s\n", sc_colcombtypes8[aA0], sc_colcombtypes8[bA0], sc_colcombtypes8[cA0], sc_colcombtypes8[dA0]);
	fprintf(fh, "\t\t//aRGB1: (%s - %s) * %s + %s\n", sc_colcombtypes16[aRGB1], sc_colcombtypes16[bRGB1], sc_colcombtypes32[cRGB1], sc_colcombtypes8[dRGB1]);		
	fprintf(fh, "\t\t//aA1  : (%s - %s) * %s + %s\n", sc_colcombtypes8[aA1],  sc_colcombtypes8[bA1], sc_colcombtypes8[cA1],  sc_colcombtypes8[dA1]);

}

//*****************************************************************************
//
//*****************************************************************************
void	PSPRenderer::BodgeBlendMode( u32 num_cycles )
{
#ifdef DAEDALUS_DEBUG_DISPLAYLIST
	mRecordedCombinerStates.insert( gRDPMux._u64 );

	if(IsCombinerStateDisabled( gRDPMux._u64 ))
	{
		// Use the nasty placeholder texture
		sceGuEnable(GU_TEXTURE_2D);
		SelectPlaceholderTexture( PTT_SELECTED );
		sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
		sceGuTexMode(GU_PSM_8888,0,0,GL_TRUE);		// maxmips/a2/swizzle = 0
		return;
	}
#endif

/*
  * Available apply-modes are: (TFX)
  *   - GU_TFX_MODULATE -			Cv=Ct*Cf					TCC_RGB: Av=Af	TCC_RGBA: Av=At*Af
  *   - GU_TFX_DECAL -	TCC_RGB:	Cv=Ct,Av=Af									TCC_RGBA: Cv=Cf*(1-At)+Ct*At Av=Af
  *   - GU_TFX_BLEND -				Cv=(Cf*(1-Ct))+(Cc*Ct)		TCC_RGB: Av=Af	TCC_RGBA: Av=At*Af
  *   - GU_TFX_REPLACE -			Cv=Ct						TCC_RGB: Av=Af	TCC_RGBA: Av=At
  *   - GU_TFX_ADD -				Cv=Cf+Ct					TCC_RGB: Av=Af	TCC_RGBA: Av=At*Af
  *

  *   - GU_TFX_BLEND -
					// Blend from Cf -> Cc using texture as the blending factor
					Cv = Cf + Ct(Cc - Cf) 
					Av = Af						Av = At*Af
*/
	bool	found( true );
	bool	install_texture( false );

	mSetDiffuseMask = 0;
	mSetDiffuseColour = 0xffffffff;
	mModulateDiffuseMask = 0;
	mModulateDiffuseColour = 0xffffffff;

	// Local vars for now
	u32	set_texture_mask = 0;
	u32 set_texture_colour = 0xffffffff;

/*
		// BK
		// XXXX Review
		case 0x0062fe043f15f9ffLL:
            //aRGB0: (1            - Primitive   ) * Env          + Primitive   
            //aA0  : (0            - 0           ) * 0            + Shade       
            //aRGB1: (Combined     - 0           ) * Shade        + 0           
            //aA1  : (Combined     - 0           ) * Env          + 0          
			break;
*/

	switch(gRDPMux._u64)
	{

		case 0x00119623ff2fffffLL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (Texel0       - 0           ) * Primitive    + 0           
		//aA1  : (Texel0       - 0           ) * Primitive    + 0  
			// Modulate the texture*primitive for RGBA
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;

			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// RgRts
		case 0x0011b223ff67ffffLL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (Texel0       - 0           ) * Primitive    + 0           
		//aA1  : (Primitive    - 0           ) * Texel0       + 0           
			// Modulate the texture*primitive for RGBA
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;

			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AL
		case 0x0011fffffffff238LL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Modulate the texture*primitive for RGB, T0 for A
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;		// Set alpha to 1.0
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		case 0x001197fffffffe38LL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// WR
		case 0x0011c223ff87ffffLL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (Shade        - 0           ) * Texel0       + 0           
		//aRGB1: (Texel0       - 0           ) * Primitive    + 0           
		//aA1  : (Shade        - 0           ) * Texel0       + 0           
			// Modulate the texture*prim for RGB, text*shade for A
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGB;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;			

		case 0x0011fe23fffff3f9LL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Texel0       - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Texel0  
			// Modulate the texture*primitive for RGB, texture for alpha
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwPrimitiveColor | COL32_MASK_A);		// 1.0 for Alpha
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		case 0x0011fe23fffff7fbLL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (Texel0       - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Primitive   
			// Modulate the texture*primitive for RGB, primitive for alpha
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// OoT
		case 0x0011fe23fffffdf8LL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Texel0       - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;		// 1.0 for alpha
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// OoT
		case 0x0011fffffffff638LL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined  
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);	// _RGB - select fragment alpha
			break;

		case 0x0011fffffffffc38LL:
		//aRGB0: (Texel0       - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;		// Set alpha to 1.0
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);	// _RGB - select fragment alpha
			break;

		// XWP
		case 0x00121624ff2fffffLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (Texel0       - 0           ) * Shade        + 0           
		//aA1  : (Texel0       - 0           ) * Primitive    + 0           
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// SFR2049
		case 0x001217fffffffe38LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT
		case 0x00121803ff0fffffLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (Combined     - 0           ) * Primitive    + 0   
			install_texture = true;
			if( num_cycles == 1 )
			{
				// Nothing to do
			}
			else
			{
				mModulateDiffuseMask = COL32_MASK_RGBA;
				mModulateDiffuseColour = m_dwPrimitiveColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// PM
		case 0x00121803ffffff78LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0
		//aA0  : (Texel0       - 0           ) * Shade        + 0
		//aRGB1: (Combined     - 0           ) * Primitive    + Env
		//aA1  : (0            - 0           ) * 0            + Combined
			install_texture = true;
			if( num_cycles == 1 )
			{
				// Ok as is
			}
			else
			{
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;

				// XXXX need to add Env
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// PM
		case 0x00121803fffffff8LL:
        //aRGB0: (Texel0       - 0           ) * Shade        + 0
        //aA0  : (Texel0       - 0           ) * Shade        + 0
        //aRGB1: (Combined     - 0           ) * Primitive    + 0
        //aA1  : (0            - 0           ) * 0            + Combined
			install_texture = true;
			if( num_cycles == 1 )
			{
				// Ok as is
			}
			else
			{
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		case 0x00121824ff33ffffLL:
		//RGB0: (Texel0       - 0           ) * Shade        + 0           
		//A0  : (Texel0       - 0           ) * Shade        + 0           
		//RGB1: (Texel0       - 0           ) * Shade        + 0           
		//A1  : (Texel0       - 0           ) * Shade        + 0 
			// Modulate the texture*shade for RGBA
			install_texture = true;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AFA
		case 0x001219fffffffe38LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Modulate the texture*shade for RGBA
			install_texture = true;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		case 0x00121a24ff37ffffLL:
		//RGB0: (Texel0       - 0           ) * Shade        + 0           
		//A0  : (Texel0       - 0           ) * Env          + 0           
		//RGB1: (Texel0       - 0           ) * Shade        + 0           
		//A1  : (Texel0       - 0           ) * Env          + 0 
			// XXXX think this is ok now
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;

			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// THPS
		case 0x00121a44fffffe38LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (Texel0       - 0           ) * Env          + 0           
		//aRGB1: (Texel1       - 0           ) * Shade        + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;

				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			}
			else
			{
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;

				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);

				// XXXX Need to add T1*Shade
			}
			break;

		// THPS
		case 0x00127244ffffec38LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 1           ) * Texel0       + 1           
		//aRGB1: (Texel1       - 0           ) * Shade        + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			// XXXX incorrect - needs 1 - T0 for alpha, +T1*Shade for 2nd cycle
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = 1-T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade + T1 * Shade
				// A   = 1-T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);			// RGBA - modulate fragment alpha
			break;
	

		// OoT drawbridge chains
		case 0x00127e03ff0ff3ffLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (Combined     - 0           ) * Primitive    + 0           
			install_texture = true;
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A   = T0 * Primitive
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwPrimitiveColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);			// RGBA - modulate fragment alpha
			break;

		// Qst64
		case 0x00127e03fffff378LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Combined     - 0           ) * Primitive    + Env         
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Primitive + Env
				// A   = T0
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			// XXXX Need to add in env
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);			// RGBA - modulate fragment alpha
			break;

		// OoT background - mountains?
		case 0x00127e03fffff3f8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A   = T0
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);			// RGBA - use texture alpha (diffuse is set to 1)
			break;

		// OoT drawbridge
		case 0x00127e03fffff9fbLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Primitive   
			install_texture = true;
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = Shade
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A   = Primitive
				mModulateDiffuseMask = COL32_MASK_RGBA;
				mModulateDiffuseColour = m_dwPrimitiveColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);			// RGB - use fragment alpha
			break;

		// OoT characters
		case 0x00127e03fffffbf8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = Env
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A   = Env
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);			// RGB - use fragment alpha of 1.0
			break;

		// OoT horse
		case 0x00127e03fffffdf8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				// RGB = T0 * Shade
				// A   = 1
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A   = 1
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);			// RGB - use fragment alpha of 1.0
			break;

		// OoT
		case 0x00127e05fffffbf8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Combined     - 0           ) * Env          + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A   = Env
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			else
			{
				// RGB = T0 * Shade * Env
				// A   = Env
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwEnvColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// OoT
		case 0x00127e05fffffdf8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Combined     - 0           ) * Env          + 0           
		//aA1  : (0            - 0           ) * 0            + Combined  
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A   = 1
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
			}
			else
			{
				// RGB = T0 * Shade * Env
				// A   = 1
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwEnvColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// OoT
		case 0x00127e60f5fffd78LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Primitive    - Env         ) * Combined     + Env         
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;

			// XXXXXXXXXXXXXXXXXXXXXXXXXX would be good to find out what this is!

			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A   = 1
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			}
			else
			{
				// RGB = Blend(Env, Prim, T0 * Shade)
				// A   = 1
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwEnvColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;

				// XXXX blend is blending texture, ignoring shade

				sceGuTexEnvColor( m_dwPrimitiveColor );
				sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGB);
			}
			break;

		case 0x00127e24fffff3f9LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Texel0       - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Texel0      
			// Modulate the texture*shade for RGB, select shade for alpha
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = MAKE_COLOUR_U32( 0,0,0, 255 );		// Override the diffuse alpha with 1.0

			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// XXXX need to come up with something more cunning for this :(
		case 0x00127e2433fdf8fcLL:
		//aRGB0: (Texel0       - Primitive   ) * Shade        + Primitive   
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (Texel0       - Primitive   ) * Shade        + Primitive   
		//aA1  : (0            - 0           ) * 0            + Shade       
			// Just select texture RGB and shade A for now
			install_texture = true;
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);
			break;

		// AM:SH
		case 0x00127e24fffff7fbLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (Texel0       - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Primitive   
			// Modulate the texture*shade for RGB, select prim for alpha
			install_texture = true;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);

			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			break;

		case 0x00127e24fffff9fcLL:
		//RGB0: (Texel0       - 0           ) * Shade        + 0           
		//A0  : (0            - 0           ) * 0            + Shade       
		//RGB1: (Texel0       - 0           ) * Shade        + 0           
		//A1  : (0            - 0           ) * 0            + Shade 
			// Modulate the texture*shade for RGB, select shade for alpha
			install_texture = true;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		case 0x00127e24fffffbfdLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Texel0       - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Env         
			// Modulate the texture*shade for RGB, select env for alpha
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// SpM
		case 0x00127e44fffff238LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Texel1       - 0           ) * Shade        + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade + T1 * Shade
				// A = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0

				// XXXX Needs T1
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT - characters
		case 0x00127e60ff17f3ffLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Primitive    - 0           ) * Combined     + 0           
		//aA1  : (Combined     - 0           ) * Env          + 0  
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A = T0 * Env
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT?
		case 0x00127e60fffff3f8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Primitive    - 0           ) * Combined     + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A   = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Primitive
				// A   = T0
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT
		case 0x00127ea0ff17f3ffLL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Env          - 0           ) * Combined     + 0           
		//aA1  : (Combined     - 0           ) * Env          + 0        
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A   = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Env
				// A   = T0 * Env
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwEnvColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT?
		case 0x00127ea0fffff3f8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Env          - 0           ) * Combined     + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if(num_cycles == 1)
			{
				// RGB = T0 * Shade
				// A   = T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			else
			{
				// RGB = T0 * Shade * Env
				// A   = T0
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwEnvColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;		// Set alpha to 1.0
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// Qst64
		case 0x00127ec5fffff238LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (1            - 0           ) * Env          + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined
			install_texture = true;
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			}
			else
			{
				// RGB	= T0 * Shade + Env
				// A	= T0
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;
				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);

				// XXXX needs to add in Env to result
			}
			break;

		// AG
		case 0x00127ffffffff238LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Modulate the texture*shade for RGB, select texture alpha
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// M64
		case 0x00127ffffffff838LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Modulate the texture*shade for RGB, select shade for alpha
			install_texture = true;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// THPS
		case 0x00127ffffffffc38LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined   
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;					// Alpha of 1.0
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// BK
		case 0x001298043f15ffffLL:
		//aRGB0: (Texel0       - Primitive   ) * Env          + Primitive   
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (Combined     - 0           ) * Env          + 0      
			install_texture = true;
			// RGB = Blend( Prim, T0, Env ) * Shade
			// A   = T0 * Shade * Env

			// Assume this is only ever used in 2 cycle
			
			// Hoplessly wrong - do Prim * Shade * T0, Env * Shade * T0
			install_texture = true;
			//mModulateDiffuseMask = COL32_MASK_RGBA;
			//mModulateDiffuseColour = (m_dwPrimitiveColor & COL32_MASK_RGB) | (m_dwEnvColor & COL32_MASK_A);

			install_texture = true;
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_RGB;
				mSetDiffuseColour = COL32_MASK_RGB;		// Set RGB to 1.0, i.e. select Texture
			}
			else
			{
				// Leave RGB shade untouched

				mModulateDiffuseMask = COL32_MASK_A;
				mModulateDiffuseColour = m_dwEnvColor;
			}

			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);

			break;

		// WR
		case 0x00129825ff33ffffLL:
		//aRGB0: (Texel0       - 0           ) * Env          + 0           
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (Texel0       - 0           ) * Env          + 0           
		//aA1  : (Texel0       - 0           ) * Shade        + 0           
			// Modulate the texture*env for RGB, text*shade for A
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGB;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		case 0x00129a25ff37ffffLL:
		//aRGB0: (Texel0       - 0           ) * Env          + 0           
		//aA0  : (Texel0       - 0           ) * Env          + 0           
		//aRGB1: (Texel0       - 0           ) * Env          + 0           
		//aA1  : (Texel0       - 0           ) * Env          + 0           
			// Modulate the texture*env for RGBA
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// MKRT
		case 0x0012fe25fffff3f9LL:
		//aRGB0: (Texel0       - 0           ) * Env          + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Texel0       - 0           ) * Env          + 0           
		//aA1  : (0            - 0           ) * 0            + Texel0      
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor | COL32_MASK_A;		// Need to set alpha to 1.0 as it's modulated with t0
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT
		case 0x0012fe25fffffdfeLL:
		//aRGB0: (Texel0       - 0           ) * Env          + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Texel0       - 0           ) * Env          + 0           
		//aA1  : (0            - 0           ) * 0            + 1           
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor | COL32_MASK_A;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);				// _RGB - use 1.0 for alpha
			break;

		// THPS
		case 0x0012fffffffffa38LL:
		//aRGB0: (Texel0       - 0           ) * Env          + 0           
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);				// _RGB - use fragment alpha
			break;

		// M64 eyes decal.
		case 0x00147e2844fe7b3dLL:
		//aRGB0: (Texel0       - Shade       ) * Texel0_Alp   + Shade       
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Texel0       - Shade       ) * Texel0_Alp   + Shade       
		//aA1  : (0            - 0           ) * 0            + Env         
			// RGB = Blend( Shade, T0, T0Alpha )  -- essentially DECAL mode on PSP
			// A   = Env
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexEnvColor( m_dwEnvColor );
			sceGuTexFunc(GU_TFX_DECAL,GU_TCC_RGBA);		
			break;

		// F0
		case 0x00147e045ffefbf8LL:
		//aRGB0: (Texel0       - Env         ) * Texel0_Alp   + Env         
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Combined 
			// RGB = Blend(E,T0,T0a)*Shade	~= Blend(SE, ST0, T0a)
			// A   = Env

			// XXXX needs to modulate again by shade
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexEnvColor( m_dwEnvColor );
			sceGuTexFunc(GU_TFX_DECAL,GU_TCC_RGBA);		
			break;

		// OoT intro
		case 0x00167e6035fcff7eLL:
        //aRGB0: (Texel0       - Primitive   ) * Env_Alpha    + Texel0      
        //aA0  : (0            - 0           ) * 0            + 0           
        //aRGB1: (Primitive    - Env         ) * Combined     + Env         
        //aA1  : (0            - 0           ) * 0            + 1           
			// XXXX incorrect, need env and prim, blend has wrong source
			install_texture = true;

			// Use the primitive for the r,g,b, override the alpha with 1.0
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (MAKE_COLOUR_U32( 0,0,0, 255) & COL32_MASK_A) | (m_dwPrimitiveColor & ~COL32_MASK_A);

			sceGuTexEnvColor( m_dwEnvColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGB);
			break;

		// OoT intro
		case 0x00267e60350cf37fLL:
        //aRGB0: (Texel1       - Primitive   ) * Env_Alpha    + Texel0      
        //aA0  : (0            - 0           ) * 0            + Texel0      
        //aRGB1: (Primitive    - Env         ) * Combined     + Env         
        //aA1  : (Combined     - 0           ) * Primitive    + 0           
			// XXXX need 2nd texture
			install_texture = true;

			// XXXX need to blend using env

			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AFA
		case 0x0026a0041f1093ffLL:
		//aRGB0: (Texel1       - Texel0      ) * LOD_Frac     + Texel0      
		//aA0  : (Texel1       - Texel0      ) * Combined     + Texel0      
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (Combined     - 0           ) * Shade        + 0           
			// XXXX need 2nd texture
			install_texture = true;
			if( num_cycles == 1 )
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			else
				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);

			break;

		// THPS
		case 0x0026a0041ffc93f8LL:
		//aRGB0: (Texel1       - Texel0      ) * LOD_Frac     + Texel0      
		//aA0  : (Texel1       - Texel0      ) * Combined     + Texel0      
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			// XXXX need 2nd texture
			install_texture = true;
			if( num_cycles == 1 )
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			else
			{
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;			// Set alpha to 1.0
				sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			}
			break;

		// AM:SH
		case 0x00309661442e7f3fLL:
		//aRGB0: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA1  : (Texel0       - 0           ) * Primitive    + 0     
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(s,p,t) for RGB, Af*At for alpha
			break;

		// XWP
		case 0x00309661552efb7dLL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Texel0       - 0           ) * Primitive    + Env         
		//aRGB1: (Primitive    - Env         ) * Texel0       + Env         
		//aA1  : (Texel0       - 0           ) * Primitive    + Env         
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive + Env
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(e,p,t) for RGB, Modulate(t,p) for alpha

			// XXXX need to add in Env for alpha
 			break;


		// MKRT ingame - smoke puffs, OoT logo text
		case 0x00309661552eff7fLL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (Primitive    - Env         ) * Texel0       + Env         
		//aA1  : (Texel0       - 0           ) * Primitive    + 0           
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(e,p,t) for RGB, Modulate(t,p) for alpha
 			break;

		// OoT - particles
		case 0x003097ff5ffefe38LL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(e,p,t) for RGB, Modulate(t,p) for alpha
			break;

		// OoT
		case 0x003097fffffffe38LL:
		//aRGB0: (Primitive    - 0           ) * Texel0       + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// SpM
		case 0x003099ff5ffefe38LL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined   
			// Blend the texture*shade for RGB, 
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGB;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);
			break;

		case 0x0030b26144664924LL:
		//aRGB0: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA0  : (Primitive    - Shade       ) * Texel0       + Shade       
		//aRGB1: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA1  : (Primitive    - Shade       ) * Texel0       + Shade       
			// Modulate the texture*shade for RGBA
			install_texture = true;
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGB);			// XXXX Argh - need to interpolate alpha too!? We're just selecting Shade for now
			break;

		// OoT intro - moon
		case 0x0030b2615566ff7fLL:
        //aRGB0: (Primitive    - Env         ) * Texel0       + Env         
        //aA0  : (Primitive    - 0           ) * Texel0       + 0           
        //aRGB1: (Primitive    - Env         ) * Texel0       + Env         
        //aA1  : (Primitive    - 0           ) * Texel0       + 0           
			// Colour is similar to the above, but we force diffuse to be env colour
			// RGB = Blend( Env, Prim, T0 )
			// A   = Prim * To
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);

			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend RGB, Modulate for alpha
			break;

		// PM
		case 0x0030b261ff2fffffLL:
        //aRGB0: (Primitive    - 0           ) * Texel0       + 0
        //aA0  : (Primitive    - 0           ) * Texel0       + 0
        //aRGB1: (Primitive    - 0           ) * Texel0       + 0
        //aA1  : (Texel0       - 0           ) * Primitive    + 0
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// SpM
		case 0x0030b3ff5ffeda38LL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Primitive    - Env         ) * Texel0       + Env         
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);
			sceGuTexEnvColor( m_dwPrimitiveColor );
			// XXXX Needs to BLEND alpha values!
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(e,p,t) for RGB, Modulate(t,p) for alpha


		// OoT rain splashes
		case 0x0030b3ff5ffefe38LL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(e,p,t) for RGB, Modulate(t,p) for alpha
			break;

		// SpM
		case 0x0030b3fffffffe38LL:
		//aRGB0: (Primitive    - 0           ) * Texel0       + 0           
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined   
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AM:SH
		case 0x0030fe6144fe7339LL:
		//aRGB0: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA1  : (0            - 0           ) * 0            + Texel0  
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;				// Set alpha to 1.0
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(s,p,t) for RGB, At*1 for alpha
			break;

		// AM:SH
		case 0x0030fe6144fe773bLL:
		//aRGB0: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (Primitive    - Shade       ) * Texel0       + Shade       
		//aA1  : (0            - 0           ) * 0            + Primitive   
			install_texture = true;
			// RGB = Blend( Env, Primitive, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexEnvColor( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGB);			// _RGBA = blend(s,p,t) for RGB, Af for alpha
			break;

		// MKRT opening (flag)
		case 0x00327e64fffff9fcLL:
		//aRGB0: (Primitive    - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (Primitive    - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Shade    
			mModulateDiffuseMask = COL32_MASK_RGB;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			break;

		// AG
		case 0x00327e64fffffbfdLL:
		//aRGB0: (Primitive    - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Primitive    - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Env         
			mModulateDiffuseMask = COL32_MASK_RGB;
			mModulateDiffuseColour = m_dwPrimitiveColor;

			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;
			break;

		// Qst64
		case 0x00327ec5fffffa38LL:
		//aRGB0: (Primitive    - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (1            - 0           ) * Env          + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			if( num_cycles == 1 )
			{
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;
			}
			else
			{
				// RGB	= Prim * Shade + Env
				// A	= Env
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwEnvColor;

				// XXXX needs to add in Env
			}
			break;

		// WR
		case 0x00327ffffffff838LL:
		//aRGB0: (Primitive    - 0           ) * Shade        + 0           
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			mModulateDiffuseMask = COL32_MASK_RGB;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			break;

		// TSph
		case 0x00323864ff73ffffLL:
		//aRGB0: (Primitive    - 0           ) * Shade        + 0           
		//aA0  : (Primitive    - 0           ) * Shade        + 0           
		//aRGB1: (Primitive    - 0           ) * Shade        + 0           
		//aA1  : (Primitive    - 0           ) * Shade        + 0           
			mModulateDiffuseMask = COL32_MASK_RGBA;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			break;

		// Qst64
		case 0x0032ffff1ffff6fbLL:
        //aRGB0: (Primitive    - Texel0      ) * Env          + 0           
        //aA0  : (0            - 0           ) * 0            + Primitive   
        //aRGB1: (0            - 0           ) * 0            + Primitive   
        //aA1  : (0            - 0           ) * 0            + Primitive   
			found = false;
			break;

		case 0x0041c683ff8fffffLL:
		//aRGB0: (Shade        - 0           ) * Primitive    + 0           
		//aA0  : (Shade        - 0           ) * Primitive    + 0           
		//aRGB1: (Shade        - 0           ) * Primitive    + 0           
		//aA1  : (Shade        - 0           ) * Primitive    + 0           
			mModulateDiffuseMask = COL32_MASK_RGBA;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			break;

		// OoT
		case 0x0041c7fffffffe38LL:
		//aRGB0: (Shade        - 0           ) * Primitive    + 0           
		//aA0  : (Shade        - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			mModulateDiffuseMask = COL32_MASK_RGBA;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			break;

		case 0x0041fffffffffc38LL:
		//aRGB0: (Shade        - 0           ) * Primitive    + 0           
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
            mModulateDiffuseMask = COL32_MASK_RGB;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A; // Alpha is 1.0
			break;

		case 0x0042ca85ff97ffffLL:
		//aRGB0: (Shade        - 0           ) * Env          + 0           
		//aA0  : (Shade        - 0           ) * Env          + 0           
		//aRGB1: (Shade        - 0           ) * Env          + 0           
		//aA1  : (Shade        - 0           ) * Env          + 0           
			mModulateDiffuseMask = COL32_MASK_RGBA;
			mModulateDiffuseColour = m_dwEnvColor;
			break;

		// AM:SH
		case 0x005096a1332dfeffLL:
		//aRGB0: (Env          - Primitive   ) * Texel0       + Primitive   
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (Env          - Primitive   ) * Texel0       + Primitive   
		//aA1  : (Texel0       - 0           ) * Primitive    + 0      
			install_texture = true;
			// RGB = Blend( Primitive, Env, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexEnvColor( m_dwEnvColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(p,e,t) for RGB, Modulate(t,p) for alpha
			break;

		// THPS
		case 0x005094023f15ffffLL:
		//aRGB0: (Env          - Primitive   ) * Texel0       + Primitive   
		//aA0  : (Texel0       - 0           ) * Texel1       + 0           
		//aRGB1: (Combined     - 0           ) * Texel1       + 0           
		//aA1  : (Combined     - 0           ) * Env          + 0           
			install_texture = true;
			// XXXX Needs T1
			if( num_cycles == 1 )
			{
				// RGB = Blend( Primitive, Env, T0 )
				// A   = T0 * T1
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;
				sceGuTexEnvColor( m_dwEnvColor );
				sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(p,e,t) for RGB, T0*1 for alpha
			}
			else
			{
				// RGB = Blend( Primitive, Env, T0 )
				// A   = T0 * T1 * Env
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = (m_dwPrimitiveColor & COL32_MASK_RGB) | (m_dwEnvColor & COL32_MASK_A);
				sceGuTexEnvColor( m_dwEnvColor );
				sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(p,e,t) for RGB, T0*Env for alpha
			}
			break;

		// THPS
		case 0x0050ffff3ffdf238LL:
		//aRGB0: (Env          - Primitive   ) * Texel0       + Primitive   
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			// RGB = Blend( Primitive, Env, T0 )
			// A   = T0 * 1
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;
			sceGuTexEnvColor( m_dwEnvColor );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(p,e,t) for RGB, T0*1 for alpha
			break;

		// SpM
		case 0x0060ffff5ffef238LL:
		//aRGB0: (1            - Env         ) * Texel0       + Env         
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			// RGB = Blend( Env, 1, T0 )
			// A   = T0
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor | COL32_MASK_A;		// 1.0 for alpha
			sceGuTexEnvColor( COL32_WHITE );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);

			break;

		// OoT, selected option
		case 0x006196c3ff2fffffLL:
		//aRGB0: (1            - 0           ) * Primitive    + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (1            - 0           ) * Primitive    + 0           
		//aA1  : (Texel0       - 0           ) * Primitive    + 0           
			// RGB = Prim
			// A   = Prim * T0
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AFA
		case 0x00627fff1ffcfc38LL:
		//aRGB0: (1            - Texel0      ) * Shade        + Texel0      
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// RGB = Blend( T0, 1, Shade )
			// A   = 1
			install_texture = true;

			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;

			// XXXX this is just selecting the texture colour, ignoring the blend
			// Could do this by manually shading texture...

			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);
			break;

		// BK
		case 0x0062fe043f15f9ffLL:
		//aRGB0: (1            - Primitive   ) * Env          + Primitive   
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (Combined     - 0           ) * Env          + 0           
		{
			u32		blend( COL32_BLEND( m_dwPrimitiveColor, MAKE_COLOUR_U32(255,255,255), m_dwEnvColor ) );

			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_RGB;
				mSetDiffuseColour = blend;
			}
			else
			{
				mModulateDiffuseMask = COL32_MASK_RGBA;
				mModulateDiffuseColour = (blend & COL32_MASK_RGB) | (m_dwEnvColor & COL32_MASK_A);
			}
		}
			break;

		// XWP
		case 0x00f0b3e13365feffLL:
		//aRGB0: (0            - Primitive   ) * Texel0       + Primitive   
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (0            - Primitive   ) * Texel0       + Primitive   
		//aA1  : (Primitive    - 0           ) * Texel0       + 0      
			install_texture = true;
			// RGB = Blend( Primitive, 0, T0 )
			// A   = T0 * Primitive
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexEnvColor( COL32_BLACK );
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);			// _RGBA = blend(p,e,t) for RGB, T0*Prim for alpha
			break;

		// XWP
		case 0x00ff97ffff2cfa7dLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (Texel0       - 0           ) * Primitive    + Env         
		//aRGB1: (0            - 0           ) * 0            + Texel0      
		//aA1  : (Texel0       - 0           ) * Primitive    + Env         
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_RGB;	// Set RGB components to white

			// XXXX need to add in Env to alpha
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// PM
		case 0x00ff97ffff2cfe7fLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0
		//aA0  : (Texel0       - 0           ) * Primitive    + 0
		//aRGB1: (0            - 0           ) * 0            + Texel0
		//aA1  : (Texel0       - 0           ) * Primitive    + 0
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_RGB;	// Set RGB components to white
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT press start
		case 0x00ff97ffff2dfeffLL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Primitive   
		//aA1  : (Texel0       - 0           ) * Primitive    + 0           
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AL
		case 0x00ff97fffffcfe38LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// RGB = T0
			// A   = T0 * Prim
			// XXXX placeholder implementation - think might be ok
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = COL32_MASK_RGB | m_dwPrimitiveColor;		// Set diffuse RGB to white
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT?
		case 0x00ff97fffffdfe38LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// RGB = Prim
			// A   = T0 * Prim
			// XXXX placeholder implementation - think might be ok
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// BM
		case 0x00ff99ffff30fe7fLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (0            - 0           ) * 0            + Texel0      
		//aA1  : (Texel0       - 0           ) * Shade        + 0           
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;


		// MKRT selection reticule
		case 0x00ff99ffff327f3fLL:
        //aRGB0: (0            - 0           ) * 0            + Shade       
        //aA0  : (Texel0       - 0           ) * Shade        + 0           
        //aRGB1: (0            - 0           ) * 0            + Shade       
        //aA1  : (Texel0       - 0           ) * Shade        + 0           
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// Qst64
		case 0x00ff99fffffffe38LL:
		//aRGB0: (0            - 0           ) * 0            + 0           
		//aA0  : (Texel0       - 0           ) * Shade        + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGB;
			mSetDiffuseColour = COL32_BLACK;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		case 0x00ff9bffff34fe7fLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (Texel0       - 0           ) * Env          + 0           
		//aRGB1: (0            - 0           ) * 0            + Texel0      
		//aA1  : (Texel0       - 0           ) * Env          + 0           
			// Modulate the texture*env for A, select texture for RGB
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			// ABGR(env,255,255,255)
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_A) | (MAKE_COLOUR_U32(255,255,255) & ~COL32_MASK_A);
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// WR
		case 0x00ff9bffff36ff7fLL:
		//aRGB0: (0            - 0           ) * 0            + Env         
		//aA0  : (Texel0       - 0           ) * Env          + 0           
		//aRGB1: (0            - 0           ) * 0            + Env         
		//aA1  : (Texel0       - 0           ) * Env          + 0           
			// Modulate the texture*env for A, select env for RGB
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// Qst64
		case 0x00ff9bfffffcfe38LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (Texel0       - 0           ) * Env          + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// RGB:	T0
			// A:	T0 * Env
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor | COL32_MASK_RGB;			// Set RGB to white
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// Qst64
		case 0x00ff9bfffffefe38LL:
		//aRGB0: (0            - 0           ) * 0            + Env         
		//aA0  : (Texel0       - 0           ) * Env          + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// RGB:	Env
			// A:	T0 * Env
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;							// Set texture to white
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// PM
		case 0x00ffb3ffff2fffffLL:
        //aRGB0: (0            - 0           ) * 0            + 0
        //aA0  : (Primitive    - 0           ) * Texel0       + 0
        //aRGB1: (0            - 0           ) * 0            + 0
        //aA1  : (Texel0       - 0           ) * Primitive    + 0
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor & COL32_MASK_A;	// Set RGB components to black
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// PM
		case 0x00ffb3ffff64fe7fLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0
		//aA0  : (Primitive    - 0           ) * Texel0       + 0
		//aRGB1: (0            - 0           ) * 0            + Texel0
		//aA1  : (Primitive    - 0           ) * Texel0       + 0
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_RGB;	// Set RGB components to white
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// F0
		case 0x00ffb3ffff65feffLL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (0            - 0           ) * 0            + Primitive   
		//aA1  : (Primitive    - 0           ) * Texel0       + 0           
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AM:SH
		case 0x00ffc7ffff8e7f3fLL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (Shade        - 0           ) * Primitive    + 0           
		//aRGB1: (0            - 0           ) * 0            + Shade       
		//aA1  : (Shade        - 0           ) * Primitive    + 0           
			mModulateDiffuseMask = COL32_MASK_A;
			mModulateDiffuseColour = m_dwPrimitiveColor;
			break;

		// RgRts
		case 0x00fffe03ff0e79ffLL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (Combined     - 0           ) * Primitive    + 0   
			break;
			if( num_cycles > 1 )
			{
				mModulateDiffuseMask = COL32_MASK_RGBA;
				mModulateDiffuseColour = m_dwPrimitiveColor;
			}
			break;

		// PM
		case 0x00fffe03fffe79f8LL:
        //aRGB0: (0            - 0           ) * 0            + Shade
        //aA0  : (0            - 0           ) * 0            + Shade
        //aRGB1: (Combined     - 0           ) * Primitive    + 0
        //aA1  : (0            - 0           ) * 0            + Combined
			if( num_cycles > 1 )
			{
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
			}
			break;

		// F0
		case 0x00fffe04fffefbf8LL:
		//aRGB0: (0            - 0           ) * 0            + Env         
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			mModulateDiffuseMask = COL32_MASK_RGB;
			mModulateDiffuseColour = m_dwEnvColor;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;
			break;

		// SpM
		case 0x00fffe44fffcf238LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Texel1       - 0           ) * Shade        + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			}
			else
			{
				// RGB	= T0 + T1*Shade
				// A	= T0

				// XXXX Need to add in T1*Shade
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			}
			break;

		// SpM
		case 0x00fffe44fffcfc38LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Texel1       - 0           ) * Shade        + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;

				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);			// RGB - Select diffuse alpha
			}
			else
			{
				// RGB	= T0 + T1*Shade
				// A	= T0

				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = COL32_MASK_A;

				// XXXX Need to add in T1*Shade
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);			// RGB - Select diffuse alpha
			}
			break;

		// SpM
		case 0x00fffe44fffdf238LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (Texel1       - 0           ) * Shade        + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;

				sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA);
			}
			else
			{
				// RGB	= T0 + T1*Shade
				// A	= T0

				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;

				// XXXX Need to add in T1*Shade
				sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA);
			}
			break;

		// Qst64
		case 0x00fffec5fffcf238LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (1            - 0           ) * Env          + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			if( num_cycles == 1 )
			{
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			}
			else
			{
				// RGB	= T0 + Env
				// A	= T0
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = m_dwEnvColor | COL32_MASK_A;
				sceGuTexFunc(GU_TFX_ADD,GU_TCC_RGBA);			// ADD/RGBA modulates alphas
			}
			break;

		// Qst64
		case 0x00fffec5fffdfa38LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (1            - 0           ) * Env          + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = (m_dwPrimitiveColor & COL32_MASK_RGB) | (m_dwEnvColor & COL32_MASK_A);
			}
			else
			{
				// RGB	= Prim+Env
				// A	= Env
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = (ADD_COLOUR_RGBA(m_dwPrimitiveColor,m_dwEnvColor) & COL32_MASK_RGB) | (m_dwEnvColor & COL32_MASK_A);
			}
			break;

		// M64
		case 0x00fffffffffcf238LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Select the texture
			install_texture = true;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			break;

		case 0x00fffffffffcf279LL:
		//RGB0: (0            - 0           ) * 0            + Texel0      
		//A0  : (0            - 0           ) * 0            + Texel0      
		//RGB1: (0            - 0           ) * 0            + Texel0      
		//A1  : (0            - 0           ) * 0            + Texel0      
			// Select the texture
			install_texture = true;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			break;

		// SpM
		case 0x00fffffffffcf438LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + Texel1      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Select the texture
			install_texture = true;
			// XXXX Needs T1
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			break;

		// PM
		case 0x00fffffffffcf67bLL:
        //aRGB0: (0            - 0           ) * 0            + Texel0
        //aA0  : (0            - 0           ) * 0            + Primitive
        //aRGB1: (0            - 0           ) * 0            + Texel0
        //aA1  : (0            - 0           ) * 0            + Primitive
			// Select the texture
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);	// _RGB - select alpha from fragment
			break;

		// AL
		case 0x00fffffffffcf83cLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Shade       
			// Select the texture
			install_texture = true;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);
			break;

		case 0x00fffffffffcf87cLL:
        //aRGB0: (0            - 0           ) * 0            + Texel0      
        //aA0  : (0            - 0           ) * 0            + Shade       
        //aRGB1: (0            - 0           ) * 0            + Texel0      
        //aA1  : (0            - 0           ) * 0            + Shade       
			// Select the texture
			install_texture = true;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);
			break;

		case 0x00fffffffffcfa7dLL:
        //aRGB0: (0            - 0           ) * 0            + Texel0      
        //aA0  : (0            - 0           ) * 0            + Env         
        //aRGB1: (0            - 0           ) * 0            + Texel0      
        //aA1  : (0            - 0           ) * 0            + Env  
			install_texture = true;

			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;

			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);		// RGB from Tex, A from Fragment
			break;

		// SpM
		case 0x00fffffffffcfc38LL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);		// RGB from Tex, A from Fragment
			break;

		// SFx
		case 0x00fffffffffcfc7eLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Texel0      
		//aA1  : (0            - 0           ) * 0            + 1           
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);		// RGB from Tex, A from Fragment
			break;


		// OoT
		case 0x00fffffffffcfe7fLL:
		//aRGB0: (0            - 0           ) * 0            + Texel0      
		//aA0  : (0            - 0           ) * 0            + 0           
		//aRGB1: (0            - 0           ) * 0            + Texel0      
		//aA1  : (0            - 0           ) * 0            + 0           
			install_texture = true;

			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = 0;		// Black, no alpha ?

			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);		// RGB from Tex, A from Fragment
			break;

		// MKRT ingame - banners
		case 0x00fffffffffdf2f9LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Primitive   
		//aA1  : (0            - 0           ) * 0            + Texel0 
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;		// ADD/RGBA modulates alpha at texture fragment
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// AL
		case 0x00fffffffffdf638LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Select just the primitive colour
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			break;

		// OoT, fade?
		case 0x00fffffffffdf6fbLL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (0            - 0           ) * 0            + Primitive   
		//aA1  : (0            - 0           ) * 0            + Primitive   
			// Select just the primitive colour
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;
			break;

		case 0x00fffffffffdfc38LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;		// Alpha is 1.0
			break;

		// XWP
		case 0x00fffffffffe773bLL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (0            - 0           ) * 0            + Primitive   
		//aRGB1: (0            - 0           ) * 0            + Shade       
		//aA1  : (0            - 0           ) * 0            + Primitive   
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			break;

		// AFA
		case 0x00fffffffffe7838LL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (0            - 0           ) * 0            + Shade       
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Select the diffuse component - this is the default
			break;

		case 0x00fffffffffe793cLL:
		//RGB0: (0            - 0           ) * 0            + Shade       
		//A0  : (0            - 0           ) * 0            + Shade       
		//RGB1: (0            - 0           ) * 0            + Shade       
		//A1  : (0            - 0           ) * 0            + Shade       
			// Select the diffuse component - this is the default
			break;

		case 0x00fffffffffe7b3dLL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (0            - 0           ) * 0            + Shade       
		//aA1  : (0            - 0           ) * 0            + Env         
			// Select the diffuse component for RGB, env colour for alpha
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwEnvColor;
			break;

		// SFR2049
		case 0x00fffffffffe7c38LL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// Select the diffuse component for RGB, 1.0 for alpha
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;
			break;

		// BM
		case 0x00fffffffffe7d3eLL:
		//aRGB0: (0            - 0           ) * 0            + Shade       
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (0            - 0           ) * 0            + Shade       
		//aA1  : (0            - 0           ) * 0            + 1    
			// Select the diffuse component for RGB, 1.0 for alpha
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;
			break;


		// AG
		case 0x00fffffffffefb7dLL:
		//aRGB0: (0            - 0           ) * 0            + Env         
		//aA0  : (0            - 0           ) * 0            + Env         
		//aRGB1: (0            - 0           ) * 0            + Env         
		//aA1  : (0            - 0           ) * 0            + Env     
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwEnvColor;
			break;

		// SpM
		case 0x00fffffffffff238LL:
		//aRGB0: (0            - 0           ) * 0            + 0           
		//aA0  : (0            - 0           ) * 0            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = COL32_MASK_A;		// RGB = 0, A = 1
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// SpM
		case 0x00fffffffffffe38LL:
		//aRGB0: (0            - 0           ) * 0            + 0           
		//aA0  : (0            - 0           ) * 0            + 0           
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined  
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = MAKE_COLOUR_U32( 0,0,0,0 );
			break;

		// BK
		case 0x00ffffffffffffffLL:
		//aRGB0: (0            - 0           ) * 0            + 0           
		//aA0  : (0            - 0           ) * 0            + 0           
		//aRGB1: (0            - 0           ) * 0            + 0           
		//aA1  : (0            - 0           ) * 0            + 0           
			// Easy :)
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = MAKE_COLOUR_U32( 0,0,0,0 );
			break;

		//
		//	Unsorted, all OoT, just after intro
		//

		// OoT skydome?
		case 0x002527ff1ffc9238LL:
		//aRGB0: (Texel1       - Texel0      ) * Prim_Alpha   + Texel0      
		//aA0  : (Texel1       - Texel0      ) * Primitive    + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// XXXX placeholder implementation - is blending between T0/T1 using primitive colour
			install_texture = true;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = MAKE_COLOUR_U32_REPLICATE_ALPHA( m_dwPrimitiveColor );
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// OoT?
		case 0x00262a60150c937fLL:
		//aRGB0: (Texel1       - Texel0      ) * Env_Alpha    + Texel0      
		//aA0  : (Texel1       - Texel0      ) * Env          + Texel0      
		//aRGB1: (Primitive    - Env         ) * Combined     + Env         
		//aA1  : (Combined     - 0           ) * Primitive    + 0           
			// XXXX placeholder implementation
			install_texture = true;
			sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
			break;

		// OoT ground texture
		case 0x00267e041ffcfdf8LL:
		//aRGB0: (Texel1       - Texel0      ) * Env_Alpha    + Texel0      
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			// XXXX placeholder implementation - ok except t1?
			install_texture = true;
			// RGB = Blend(T0, T1, EnvAlpha) * Shade
			// A   = 1
			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = COL32_MASK_A;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;



		case 0x00267e041f0cfdffLL:
		//aRGB0: (Texel1       - Texel0      ) * Env_Alpha    + Texel0      
		//aA0  : (0            - 0           ) * 0            + 1           
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (Combined     - 0           ) * Primitive    + 0           
			// XXXX placeholder implementation
			install_texture = true;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGB);
			break;

		// OoT paths
		case 0x00121603ff5bfff8LL:
		//aRGB0: (Texel0       - 0           ) * Shade        + 0           
		//aA0  : (Texel0       - 0           ) * Primitive    + 0           
		//aRGB1: (Combined     - 0           ) * Primitive    + 0           
		//aA1  : (Texel1       - 0           ) * 1            + Combined  
			// XXXX placeholder implementation - think is correct except t1
			install_texture = true;
			// RGB = T0 * Shade * Primitive
			// A   = T0 * Primitive + T1
			if( num_cycles == 1 )
			{
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwPrimitiveColor;
			}
			else
			{
				mModulateDiffuseMask = COL32_MASK_RGB;
				mModulateDiffuseColour = m_dwPrimitiveColor;
				mSetDiffuseMask = COL32_MASK_A;
				mSetDiffuseColour = m_dwPrimitiveColor;
			}
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;


		// Zelda OoT logo flames
		case 0x00272c60350ce37fLL:
		//aRGB0: (Texel1       - Primitive   ) * PrimLODFrac  + Texel0      
		//aA0  : (Texel1       - 1           ) * 1            + Texel0      
		//aRGB1: (Primitive    - Env         ) * Combined     + Env         
		//aA1  : (Combined     - 0           ) * Primitive    + 0           
			// XXXX placeholder implementation
			install_texture = true;

			if( num_cycles == 1 )
			{
				// RGB = T0 + x(T1-Prim)
				// A   = T0+T1-1
				sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);		// XXXX No T1
			}
			else
			{
				// RGB = Blend(Env, Prim, (T0 + x(t1-Prim)))
				// A   = (T0+T1-1)*Prim
				mSetDiffuseMask = COL32_MASK_RGBA;
				mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);

				sceGuTexEnvColor( m_dwPrimitiveColor );
				sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);		// XXXX No T1

			}

			break;

		case 0x0030b2045ffefff8LL:
		//aRGB0: (Primitive    - Env         ) * Texel0       + Env         
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (0            - 0           ) * 0            + Combined    
			// XXXX placeholder implementation - correct for 1 cycle?
			install_texture = true;

			// Use Env for RGB, Prim for A
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = (m_dwEnvColor & COL32_MASK_RGB) | (m_dwPrimitiveColor & COL32_MASK_A);

			sceGuTexEnvColor( m_dwPrimitiveColor );

			if( num_cycles == 1 )
			{
				// RGB = Blend(Env,Prim,T0)
				// A   = Prim * T0
			}
			else
			{
				// RGB = Blend(Env,Prim,T0) * Shade
				// A   = Prim * T0

				// Can't handle shade for second cycle :(
			}
			sceGuTexFunc(GU_TFX_BLEND,GU_TCC_RGBA);	// _RGBA modulate
			break;

		// OoT, letters
		case 0x00ffadfffffd9238LL:
		//aRGB0: (0            - 0           ) * 0            + Primitive   
		//aA0  : (Texel1       - Texel0      ) * 1            + Texel0      
		//aRGB1: (0            - 0           ) * 0            + Combined    
		//aA1  : (0            - 0           ) * 0            + Combined    
			// XXXX placeholder implementation - needs t1
			// RGB = Prim
			// A   = (T1-T0)+T0 = T1
			install_texture = true;
			set_texture_mask = COL32_MASK_RGB;
			set_texture_colour = COL32_WHITE;
			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor | COL32_MASK_A;		// Want to select texture alpha
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);
			break;

		// MKRT ingame - cars, balloons etc
		case 0x0060b2c15565feffLL:
		//aRGB0: (1            - Env         ) * Texel0       + Primitive   
		//aA0  : (Primitive    - 0           ) * Texel0       + 0           
		//aRGB1: (1            - Env         ) * Texel0       + Primitive   
		//aA1  : (Primitive    - 0           ) * Texel0       + 0           
			install_texture = true;
			//XXXX placeholder - seems okish

			mSetDiffuseMask = COL32_MASK_RGBA;
			mSetDiffuseColour = m_dwPrimitiveColor;

			//sceGuTexEnvColor( MAKE_COLOUR_U32_RGB_1MC( m_dwEnvColor ) );	// 1-r, 1-g, 1-b, a
			sceGuTexFunc(GU_TFX_ADD,GU_TCC_RGBA);			// in RGBA mode alpha = texture*diffuse fragment
			break;

		// TSph
		case 0x0026a0041f0c93ffLL:
		//aRGB0: (Texel1       - Texel0      ) * LOD_Frac     + Texel0      
		//aA0  : (Texel1       - Texel0      ) * Combined     + Texel0      
		//aRGB1: (Combined     - 0           ) * Shade        + 0           
		//aA1  : (Combined     - 0           ) * Primitive    + 0           
			install_texture = true;
			//XXXX placeholder

			mSetDiffuseMask = COL32_MASK_A;
			mSetDiffuseColour = m_dwPrimitiveColor;
			sceGuTexFunc(GU_TFX_MODULATE,GU_TCC_RGBA);			// in RGBA mode alpha = texture*diffuse fragment
			break;

		default:
			found = false;
			break;
	}

	bool	installed_texture( false );

	if(install_texture)
	{
		if(m_lpsTexture[0] != NULL)
		{
			CNativeTexture * p_texture;

			if(set_texture_mask != 0)
			{
				p_texture = m_lpsTexture[0]->GetRecolouredTexture( set_texture_mask, set_texture_colour );
			}
			else
			{
				p_texture = m_lpsTexture[0]->GetTexture();
			}

			if(p_texture != NULL)
			{
				p_texture->InstallTexture();
				installed_texture = true;
			}
		}
	}

	// If no texture was specified, or if we couldn't load it, clear it out
	if( !installed_texture )
	{
		sceGuDisable(GU_TEXTURE_2D);
	}

	if(!found)
	{
		if(mUnhandledCombinderStates.find( gRDPMux._u64 ) == mUnhandledCombinderStates.end())
		{
			char szFilePath[MAX_PATH+1];

			Dump_GetDumpDirectory(szFilePath, g_ROM.settings.szGameName);

			IO::Path::Append(szFilePath, "missing_mux.txt");

			FILE * fh( fopen(szFilePath, mUnhandledCombinderStates.empty() ? "w" : "a") );
			if(fh != NULL)
			{
				PrintMux( fh, gRDPMux._u64 );
				fclose(fh);
			}

			mUnhandledCombinderStates.insert( gRDPMux._u64 );
		}
		

		sceGuEnable(GU_TEXTURE_2D);
		sceGuTexMode(GU_PSM_8888,0,0,GL_TRUE);		// maxmips/a2/swizzle = 0

		// Use the nasty placeholder texture
		SelectPlaceholderTexture( PTT_MISSING );
		sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGBA);
	}
}

