	
	.global		_TransformVerticesWithLighting
	.global		_TransformVerticesWithColour
	
.data	
recip_255:
.align 4
	.float		0.003921568627450980392156862745098
	.float		0.003921568627450980392156862745098
	.float		0.003921568627450980392156862745098
	.float		0.003921568627450980392156862745098
.align 2

.text
	.set		push
	.set		noreorder
	
############################
#	a0 - world matrix			- must be aligned to 16 bytes
#	a1 - projection matrix		- must be aligned to 16 bytes
#	a2 - Fiddled vertices		- stride 16
#	a3 - Output vertices		- must be aligned to 16 bytes, stride 64
#	t0 - num vertices
#	t1 - ambient colour
#	t2 - lights
#	t3 - num_lights
_TransformVerticesWithLighting:
	lv.q		R000, 0($a0)		// Load mat world
	lv.q		R001, 16($a0)
	lv.q		R002, 32($a0)
	lv.q		R003, 48($a0)

	lv.q		R200, 0($a1)		// Load mat project
	lv.q		R201, 16($a1)
	lv.q		R202, 32($a1)
	lv.q		R203, 48($a1)
	
	# Produce world*project in M100
	vmmul.q		M100, M000, M200

	sll			$t0, $t0, 4			// count = count * 16
	addu		$t0, $a2, $t0		// end_ptr = start_ptr + count * 16
	
	ulv.q		R300, 0($t1)		// Load ambient into R300

next_vertex_lighting:
	beq			$a2, $t0, finished_lighting
	nop

	# Load and transform this vertex position
 	lh			$v0,2($a2)			// x (offset of 2)
 	mtv			$v0, S200
 	lh			$v1,0($a2)			// y (offset of 0)
 	mtv			$v1, S210
 	lh			$v0,6($a2)			// z (offset of 6)
 	mtv			$v0, S220
 	li			$v1,1
 	mtv			$v1, S230
 	
 	vi2f.q		R200, R200, 0		// int -> float
 
	vtfm4.q		R201, M000, R200	// World transform
	vtfm4.q		R202, M100, R200	// Projection transform
	
	sv.q		R201, 0($a3)		// Store world transform
	sv.q		R202, 16($a3)		// Store projection transform
	
	# Load the normal into R200
	lb			$v0,15($a2)			// x
	mtv			$v0, S200
	lb			$v1,14($a2)			// y
	mtv			$v1, S210
	lb			$v0,13($a2)			// z
	mtv			$v0, S220
	li			$v1, 0
	mtv			$v1, S230

	# Convert to float and transform
 	vi2f.q		R201, R200, 0				// int -> float (obliterates world transform)
	vtfm4.q		R200, M000, R201			// Transform with the world view matrix
	
	vdot.t		S201, R200, R200			// S201 = x*x + y*y + z*z
	vrsq.s		S201, S201					// S201 = 1/sqrt(x*x + y*y + z*z)
	vmul.q		R200, R200, R201[x,x,x,x]	// S200 = v.normalise()

	# Lighting calculation
	# M000: World Matrix
	# M100: Projection Matrix
	# R200: Material normal 
	# R201: Accumulated colour
	# R202: ?
	# R203: ?
	# R300: Ambient
	# R301: Light normal
	# R302: Light colour
	# R303: Scratch
	# t1 = last_light
	# t2 = p_lights
	# t3 = num_lights
	# t4 = cur_light
	
	vmov.q		R201, R300			// Colour = ambient

	sll			$t1, $t3, 5			// t1 = num_lights*32
	addu		$t1, $t2, $t1		// last_light = p_lights + num_lights*32
	or			$t4, $t2, $0		// cur_light = p_lights

next_light_2:
	beq			$t4, $t1, done_lighting		// cur_light == last_light?
	nop
	lv.q		R301, 0($t4)		// Load normal into R301
	lv.q		R302, 16($t4)		// Load colour into R302

	vdot.t		S303[0:1], R200, R301		// x = clamp(dot(normal,(x,y,z,0)),0,1)
	vmul.q		R303, R303[x,x,x,x], R302	// r,g,b,a = x*r, x*g, x*b, x*a
	vadd.q		R201, R201, R303			// col += r,g,b,a

	b			next_light_2
	addiu		$t4, $t4, 32		// Skip to the next light

done_lighting:
	vmov.q		R200[0:1,0:1,0:1,0:1], R201			// Clamp 0..1
	sv.q		R200, 32($a3)						// Store colour

	# Continue with the next vertex
	addiu		$a2, $a2, 16		// Next input vertex
	b			next_vertex_lighting
	addiu		$a3, $a3, 64		// Next output vertex

finished_lighting:	
	jr			$ra
	nop
	

############################
#	a0 - world matrix			- must be aligned to 16 bytes
#	a1 - projection matrix		- must be aligned to 16 bytes
#	a2 - Fiddled vertices		- stride 16
#	a3 - Output vertices		- must be aligned to 16 bytes, stride 64
#	t0 - num vertices
_TransformVerticesWithColour:
	nop
foo:
	lv.q		R000, 0($a0)		// Load mat world
	lv.q		R001, 16($a0)
	lv.q		R002, 32($a0)
	lv.q		R003, 48($a0)

	lv.q		R200, 0($a1)		// Load mat project
	lv.q		R201, 16($a1)
	lv.q		R202, 32($a1)
	lv.q		R203, 48($a1)
	
	# Produce world*project in M100
	vmmul.q		M100, M000, M200
	
	# Load 1/255
	la			$v0, recip_255
	lv.q		R203, 0($v0)

	sll			$t0, $t0, 4			// count = count * 16
	addu		$t0, $a2, $t0		// end_ptr = start_ptr + count * 16

next_vertex_colour:
	beq			$a2, $t0, finished_colour
	nop

	# Load and transform this vertex position
 	lh			$v0,2($a2)			// x (offset of 2)
 	mtv			$v0, S200
 	lh			$v1,0($a2)			// y (offset of 0)
 	mtv			$v1, S210
 	lh			$v0,6($a2)			// z (offset of 6)
 	mtv			$v0, S220
 	li			$v1,1
 	mtv			$v1, S230
 	
 	vi2f.q		R200, R200, 0		// int -> float
 
	vtfm4.q		R201, M000, R200	// World transform
	vtfm4.q		R202, M100, R200	// Projection transform
	
	sv.q		R201, 0($a3)		// Store world transform
	sv.q		R202, 16($a3)		// Store projection transform
	
	# Load and normalise the RGBA colour
	lbu			$v0,15($a2)			// r
	mtv			$v0, S200
	lbu			$v1,14($a2)			// g
	mtv			$v1, S210
	lbu			$v0,13($a2)			// b
	mtv			$v0, S220
	lbu			$v1,12($a2)			// a
	mtv			$v1, S230
	
	// R200 = [r,g,b,a]
 	vi2f.q		R200, R200, 0		// int -> float
	vmul.q		R200, R200, R203	// R200 = [r * 1/255, g * 1/255, b * 1/255, a * 1/255]
	sv.q		R200, 32($a3)		// Store colour

	# Continue with the next vertex
	addiu		$a2, $a2, 16		// Next input vertex
	b			next_vertex_colour
	addiu		$a3, $a3, 64		// Next output vertex
	
finished_colour:
	jr			$ra
	nop
	
	b foo
	nop
	
	.set pop

